/*
 * Created on 17.03.2004
 *
 */
package cp.text;

import java.awt.Color;
import java.io.*;
import java.util.*;
import java.net.*;
import java.nio.charset.Charset;

/**
 * @author Peter Knees
 *
 */
public class TextTool {
	
	public static String colorToHtml(Color col) {
		return "#"+	leadingZero(Integer.toHexString(col.getRed()))+
					leadingZero(Integer.toHexString(col.getGreen()))+
					leadingZero(Integer.toHexString(col.getBlue()));
	}
	
	public static Color htmlToColor(String col) {
		if (!col.matches("#[0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F][0-9a-fA-F]"))
			return Color.WHITE;
		int red		= Integer.parseInt(col.substring(1,3), 16);
		int green	= Integer.parseInt(col.substring(3,5), 16);
		int blue	= Integer.parseInt(col.substring(5,7), 16);
		return new Color(red, green, blue);
	}
	
	public static String removeUnwantedChars(String s) {
		StringBuffer sb = new StringBuffer();
		for (int i=0; i<s.length(); i++) {
			char c = s.charAt(i);
//			if (Character.isLetterOrDigit(c))
			if ("0123456789abcdefghijklmnopqrstuvwxyz".indexOf(c+"") != -1)
				sb.append(c);
		}
		return sb.toString().toLowerCase();
	}
	
	public static String leadingZero(String s) {
		if (s.length() == 1)
			s = "0" + s;
		return s;
	}
	
	public static String leadingDoubleZero(String s) {
		if (s.length() == 1)
			s = "00" + s;
		else if (s.length() == 2)
			s = "0" + s;
		return s;
	}
	
	public static String removeLeadingChars(String s, String chars) {
		int startpos = 0;
		while (startpos<s.length() && chars.contains(""+s.charAt(startpos))) startpos++;
		return s.substring(startpos);
	}
	
	public static String doubleToString(double d, int posAfterComma) {
		int factor = (int)(Math.pow(10, posAfterComma));
		String s = ""+(Math.round(d*factor)/(double)factor);
		return s.substring(0, Math.min(s.length(), s.indexOf('.')+posAfterComma+1));
	}
	
	public static void stringToFile(String content, File file) {
		stringToFile(content, file, Charset.defaultCharset().name());
	}
	
	public static void stringToFile(String content, File file, String csn) {
		try {
			PrintWriter pw = new PrintWriter(file, csn);
			pw.print(content);
			pw.flush();
			pw.close();		
		}
		catch (Exception ioe) {
			System.err.println("error: invalid file. " + ioe.getMessage());
		}
	}
	
	
	public static void appendStringToFile(String content, File file) {
		appendStringToFile(content, file, Charset.defaultCharset().name());
	}
			
	public static void appendStringToFile(String content, File file, String csn) {
		BufferedWriter bw = null;
		try{
	        
            OutputStreamWriter writer = new OutputStreamWriter(
                  new FileOutputStream(file, true), csn);
            bw = new BufferedWriter(writer);
		}
		catch (Exception ioe) {
			System.err.println("error: invalid file: "+ioe.getMessage());
			return;
		}
		try {
			PrintWriter pw = new PrintWriter(bw);
			pw.print(content);
			pw.flush();
			pw.close();		
			bw.close();
		}
		catch (Exception ioe) {
			System.err.println("error: printing or closing error.");
		}
	}
	
	public static void stringArrayToFile(String[] stringarray, File file) {
		stringArrayToFile(stringarray, file, Charset.defaultCharset().name());
	}
	
	public static void stringArrayToFile(String[] stringarray, File file, String csn) {
		StringBuffer out = new StringBuffer();
		for (int i=0; i<stringarray.length; i++) {
			out.append(stringarray[i]+"\r\n");
		}
		stringToFile(out.toString(), file);
	}
	
	public static void intArrayToFile(int[] intarray, File file) {
		intArrayToFile(intarray, file, Charset.defaultCharset().name());
	}
	
	public static void intArrayToFile(int[] intarray, File file, String csn) {
		StringBuffer out = new StringBuffer();
		for (int i=0; i<intarray.length; i++) {
			out.append(intarray[i]+"\r\n");
		}
		stringToFile(out.toString(), file, csn);
	}
	
	public static void doubleArrayToFile(double[] doublearray, File file) {
		doubleArrayToFile(doublearray, file, Charset.defaultCharset().name());
	}
	
	public static void doubleArrayToFile(double[] doublearray, File file, String csn) {
		StringBuffer out = new StringBuffer();
		for (int i=0; i<doublearray.length; i++) {
			out.append(doublearray[i]+"\r\n");
		}
		stringToFile(out.toString(), file, csn);
	}
	
	public static void floatArrayToFile(float[] floatarray, File file) {
		floatArrayToFile(floatarray, file, Charset.defaultCharset().name());
	}
	
	public static void floatArrayToFile(float[] floatarray, File file, String csn) {
		StringBuffer out = new StringBuffer();
		for (int i=0; i<floatarray.length; i++) {
			out.append(floatarray[i]+"\r\n");
		}
		stringToFile(out.toString(), file, csn);
	}
	
	public static void doubleMatrixToFile(double[][] doublematrix, File file) {
		doubleMatrixToFile(doublematrix, file, Charset.defaultCharset().name());
	}
	
	public static void doubleMatrixToFile(double[][] doublematrix, File file, String csn) {
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(file, csn);
		}
		catch (Exception ioe) {
			System.err.println("error: invalid file.");
			return;
		}
		for (int i=0; i<doublematrix.length; i++) {
			StringBuffer out = new StringBuffer();
			for (int j=0; j<doublematrix[i].length; j++) {
				if (j==doublematrix[i].length-1)
					out.append(doublematrix[i][j]+"\r\n");
				else
					out.append(doublematrix[i][j]+" ");
			}
			pw.print(out.toString());
			pw.flush();
		}
		pw.close();	
	}
	
	public static void floatMatrixToFile(float[][] floatmatrix, File file) {
		floatMatrixToFile(floatmatrix, file, Charset.defaultCharset().name());
	}

	public static void floatMatrixToFile(float[][] floatmatrix, File file, String csn) {
		PrintWriter pw;
		try {
			pw = new PrintWriter(file, csn);
		}
		catch (Exception ioe) {
			System.err.println("error: invalid file.");
			return;
		}
		for (int i=0; i<floatmatrix.length; i++) {
			StringBuffer out = new StringBuffer();
			for (int j=0; j<floatmatrix[i].length; j++) {
				if (j==floatmatrix[i].length-1)
					out.append(floatmatrix[i][j]+"\r\n");
				else
					out.append(floatmatrix[i][j]+" ");
			}
			pw.print(out.toString());
			pw.flush();
		}
		pw.close();	
	}
	public static String readInFile(File file) throws Exception {
		if (!file.canRead()) {
			throw new Exception("can't read file.");
		}
		FileReader fr;
		try {
			fr = new FileReader(file);
		}
		catch (FileNotFoundException fnfe) {
			throw new Exception("file not found.");
		}
		BufferedReader br = new BufferedReader(fr);
		StringBuffer sb = new StringBuffer();
		String line;
		try {
			while ((line = br.readLine()) != null) {
				sb.append(line + '\n');
			}
		}
		catch (IOException e) {
			throw new Exception("error when reading file.");
		}
		fr.close();
		return sb.toString();
	}
	
	public static String readInFileUTF8(File file) throws Exception {
		if (!file.canRead()) {
			throw new Exception("can't read file.");
		}
		StringBuffer sb = new StringBuffer();
		char[] buffer = new char[2048];
		
	    InputStreamReader isr = new InputStreamReader(new FileInputStream(file), Charset.forName("UTF-8"));
	    
	    int len;
	    while ((len = isr.read(buffer)) != -1) {
	    	sb.append(buffer, 0, len);
	    }
		return sb.toString();
	}
	
	public static String readInFile(File file, String charset) throws Exception {
		if (!file.canRead()) {
			throw new Exception("can't read file.");
		}
		StringBuffer sb = new StringBuffer();
		char[] buffer = new char[2048];
		
	    InputStreamReader isr = new InputStreamReader(new FileInputStream(file), Charset.forName(charset));
	    
	    int len;
	    while ((len = isr.read(buffer)) != -1) {
	    	sb.append(buffer, 0, len);
	    }
		return sb.toString();
	}

	
	public static String readInURL(URL url) throws Exception {
		// open the URL
		URLConnection urlConnection = url.openConnection();
		InputStreamReader streamRead = new InputStreamReader(url.openStream());
		BufferedReader br = new BufferedReader(streamRead);
		// read in the URL
			
		StringBuffer sb = new StringBuffer();
		String line;
		try {
			while ((line = br.readLine()) != null) {
				sb.append(line + '\n');
			}
		}
		catch (IOException e) {
			throw new Exception("error when reading from url.");
		}
		streamRead.close();
		return sb.toString();
	}
	
	public static String[] readInStringArray(File file, boolean ignorecomments) throws Exception {
		String array = readInFile(file);
		Vector strings = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken();
			if (ignorecomments && t.startsWith("%")) continue;
			if (t.trim().equals("")) continue;
			strings.addElement(t);
		}
		String[] r = new String[strings.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((String)(strings.elementAt(i)));
		}
		return r;
	}
	
	public static String[] readInStringArrayUTF8(File file, boolean ignorecomments) throws Exception {
		String array = readInFileUTF8(file);
		Vector strings = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken();
			if (ignorecomments && t.startsWith("%")) continue;
			if (t.trim().equals("")) continue;
			strings.addElement(t);
		}
		String[] r = new String[strings.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((String)(strings.elementAt(i)));
		}
		return r;
	}
	
	public static String[] readInStringArray(File file, String charset, boolean ignorecomments) throws Exception {
		String array = readInFile(file, charset);
		Vector strings = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken();
			if (ignorecomments && t.startsWith("%")) continue;
			if (t.trim().equals("")) continue;
			strings.addElement(t);
		}
		String[] r = new String[strings.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((String)(strings.elementAt(i)));
		}
		return r;
	}
	
	public static int[] readInIntArray(File file, boolean ignorecomments) throws Exception {
		String array = readInFile(file);
		Vector ints = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken().trim();
			if (ignorecomments && t.startsWith("%")) continue;
			try {
				int n = Integer.parseInt(t);
				ints.addElement(new Integer(n));
			}
			catch (NumberFormatException nfe) {
				continue;
			}
		}
		int[] r = new int[ints.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((Integer)(ints.elementAt(i))).intValue();
		}
		return r;
	}
	
	public static double[] readInDoubleArray(File file, boolean ignorecomments) throws Exception {
		String array = readInFile(file);
		Vector doubles = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken().trim();
			if (ignorecomments && t.startsWith("%")) continue;
			try {
				double n = Double.parseDouble(t);
				doubles.addElement(new Double(n));
			}
			catch (NumberFormatException nfe) {
				continue;
			}
		}
		double[] r = new double[doubles.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((Double)(doubles.elementAt(i))).doubleValue();
		}
		return r;
	}

	public static float[] readInFloatArray(File file, boolean ignorecomments) throws Exception {
		String array = readInFile(file);
		Vector<Float> floats = new Vector<Float>();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken().trim();
			if (ignorecomments && t.startsWith("%")) continue;
			try {
				float n = Float.parseFloat(t);
				floats.addElement(new Float(n));
			}
			catch (NumberFormatException nfe) {
				continue;
			}
		}
		float[] r = new float[floats.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = floats.elementAt(i).floatValue();
		}
		return r;
	}
	
	public static double[][] readInDoubleMatrix(File file) throws Exception {
		if (!file.canRead()) {
			throw new Exception("can't read file.");
		}
		FileReader fr;
		try {
			fr = new FileReader(file);
		}
		catch (FileNotFoundException fnfe) {
			throw new Exception("file not found.");
		}
		Vector<double[]> temparray = new Vector<double[]>();
		
		BufferedReader br = new BufferedReader(fr);
		String line;
		try {
			while ((line = br.readLine()) != null) {
				String t = line.trim();
				if (t.equals(""))
					continue;
				StringTokenizer linetok = new StringTokenizer(t, " ");
				double[] i_array = new double[linetok.countTokens()];
				for (int j=0; j<i_array.length && linetok.hasMoreTokens(); j++) {
					try {
						i_array[j] = Double.parseDouble(linetok.nextToken());
					}
					catch (NumberFormatException nfe) { // else 0.
					}
				}
				temparray.addElement(i_array);
			}
		}
		catch (IOException e) {
			throw new Exception("error when reading file.");
		}
		fr.close();
		
		double[][] darray = new double[temparray.size()][];
		for (int i=0; i<darray.length; i++) {
			darray[i] = temparray.get(i);
		}
		return darray;
	}
	
	public static float[][] readInFloatMatrix(File file) throws Exception {
		if (!file.canRead()) {
			throw new Exception("can't read file.");
		}
		FileReader fr;
		try {
			fr = new FileReader(file);
		}
		catch (FileNotFoundException fnfe) {
			throw new Exception("file not found.");
		}
		Vector<float[]> temparray = new Vector<float[]>();
		
		BufferedReader br = new BufferedReader(fr);
		String line;
		try {
			while ((line = br.readLine()) != null) {
				String t = line.trim();
				if (t.equals(""))
					continue;
				StringTokenizer linetok = new StringTokenizer(t, " ");
				float[] i_array = new float[linetok.countTokens()];
				for (int j=0; j<i_array.length && linetok.hasMoreTokens(); j++) {
					try {
						i_array[j] = Float.parseFloat(linetok.nextToken());
					}
					catch (NumberFormatException nfe) { // else 0.
					}
				}
				temparray.addElement(i_array);
			}
		}
		catch (IOException e) {
			throw new Exception("error when reading file.");
		}
		fr.close();
		
		float[][] farray = new float[temparray.size()][];
		for (int i=0; i<farray.length; i++) {
			farray[i] = temparray.get(i);
		}
		return farray;
	}
	
	public static String[] readInStringArray(URL url, boolean ignorecomments) throws Exception {
		String array = readInURL(url);
		Vector strings = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken();
			if (ignorecomments && t.startsWith("%")) continue;
			if (t.trim().equals("")) continue;
			strings.addElement(t);
		}
		String[] r = new String[strings.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((String)(strings.elementAt(i)));
		}
		return r;
	}
	
	public static int[] readInIntArray(URL url, boolean ignorecomments) throws Exception {
		String array = readInURL(url);
		Vector ints = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken().trim();
			if (ignorecomments && t.startsWith("%")) continue;
			try {
				int n = Integer.parseInt(t);
				ints.addElement(new Integer(n));
			}
			catch (NumberFormatException nfe) {
				continue;
			}
		}
		int[] r = new int[ints.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((Integer)(ints.elementAt(i))).intValue();
		}
		return r;
	}
	
	public static double[] readInDoubleArray(URL url, boolean ignorecomments) throws Exception {
		String array = readInURL(url);
		Vector doubles = new Vector();
		StringTokenizer st = new StringTokenizer(array, "\r\n");
		while (st.hasMoreTokens()) {
			String t = st.nextToken().trim();
			if (ignorecomments && t.startsWith("%")) continue;
			try {
				double n = Double.parseDouble(t);
				doubles.addElement(new Double(n));
			}
			catch (NumberFormatException nfe) {
				continue;
			}
		}
		double[] r = new double[doubles.size()];
		for (int i=0; i<r.length; i++) {
			r[i] = ((Double)(doubles.elementAt(i))).doubleValue();
		}
		return r;
	}

	public static String toMatlabFormat(int[] a) {
		StringBuffer sb = new StringBuffer("[");
		for (int i=0; i<a.length; i++) {
			sb.append(a[i]);
			if (i<a.length-1)
				sb.append(", ");
			else 
				sb.append("];");
		}
		return sb.toString();
	}
	
	public static String toMatlabFormat(double[] a) {
		StringBuffer sb = new StringBuffer("[");
		for (int i=0; i<a.length; i++) {
			sb.append(a[i]);
			if (i<a.length-1)
				sb.append(", ");
			else 
				sb.append("];");
		}
		return sb.toString();
	}
	
	public static String toMatlabFormat(int[][] a) {
		StringBuffer sb = new StringBuffer("[");
		for (int i=0; i<a.length; i++) {
			for (int j=0; j<a[i].length; j++) {
				sb.append(a[i][j]);
				if (j<a[i].length-1)
					sb.append(", ");
				else 
					sb.append("; ...\n");
			}
		}
		sb.append(']');
		return sb.toString();
	}
	
	public static String toMatlabFormat(float[][] a) {
		StringBuffer sb = new StringBuffer("[");
		for (int i=0; i<a.length; i++) {
			for (int j=0; j<a[i].length; j++) {
				sb.append(new Float(a[i][j]).doubleValue());
				if (j<a[i].length-1)
					sb.append(", ");
				else 
					sb.append("; ...\n");
			}
		}
		sb.append(']');
		return sb.toString();
	}
	
	public static String toMatlabFormat(double[][] a) {
		StringBuffer sb = new StringBuffer("[");
		for (int i=0; i<a.length; i++) {
			for (int j=0; j<a[i].length; j++) {
				sb.append(a[i][j]);
				if (j<a[i].length-1)
					sb.append(", ");
				else 
					sb.append("; ...\n");
			}
		}
		sb.append(']');
		return sb.toString();
	}
	
	public static String toMatlabFormat(String[] a) {
		StringBuffer sb = new StringBuffer("{");
		for (int i=0; i<a.length; i++) {
			sb.append("'"+a[i].replace("'", "''")+"'");
			if (i<a.length-1)
				sb.append("; ");
		}
		sb.append('}');
		return sb.toString();
	}
	
	public static void writeInMatlabFormat(double[][] a, File file) {
		FileWriter fw = null;
		try {
			fw = new FileWriter(file);
		}
		catch (Exception ioe) {
			System.err.println("error: invalid file.");
			return;
		}
		try {
			PrintWriter pw = new PrintWriter(fw);
			pw.print("[");
			for (int i=0; i<a.length; i++) {
				for (int j=0; j<a[i].length; j++) {
					pw.print(a[i][j]);
					if (j<a[i].length-1)
						pw.print(", ");
					else 
						pw.print("; ...\n");
				}
			}
			pw.print(']');
			pw.flush();
			pw.close();		
			fw.close();
		}
		catch (Exception ioe) {
			System.err.println("error: printing or closing error.");
		}
		
		
	}
}
